@@ -1,14 +1,15 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-import os
-import sys
-import re
 import fnmatch
+import os
+import re
+import sys
+from shutil import get_terminal_size
 
 import numpy as np
 
 from astropy import log
-from astropy.utils.console import Getch, color_print, terminal_size, conf
+from astropy.utils.console import Getch, color_print, conf
 from astropy.utils.data_info import dtype_info_name
 
 __all__ = []
@@ -16,21 +17,23 @@ __all__ = []
 
 def default_format_func(format_, val):
     if isinstance(val, bytes):
-        return val.decode('utf-8', errors='replace')
+        return val.decode("utf-8", errors="replace")
     else:
         return str(val)
 
 
 # The first three functions are helpers for _auto_format_func
 
+
 def _use_str_for_masked_values(format_func):
     """Wrap format function to trap masked values.
 
     String format functions and most user functions will not be able to deal
     with masked values, so we wrap them to ensure they are passed to str().
     """
-    return lambda format_, val: (str(val) if val is np.ma.masked
-                                 else format_func(format_, val))
+    return lambda format_, val: (
+        str(val) if val is np.ma.masked else format_func(format_, val)
+    )
 
 
 def _possible_string_format_functions(format_):
@@ -46,8 +49,8 @@ def _possible_string_format_functions(format_):
 
 
 def get_auto_format_func(
-        col=None,
-        possible_string_format_functions=_possible_string_format_functions):
+    col=None, possible_string_format_functions=_possible_string_format_functions
+):
     """
     Return a wrapped ``auto_format_func`` function which is used in
     formatting table columns.  This is primarily an internal function but
@@ -83,13 +86,14 @@ def get_auto_format_func(
             return col.info._format_funcs[format_](format_, val)
 
         if callable(format_):
-            format_func = lambda format_, val: format_(val)  # noqa
+            format_func = lambda format_, val: format_(val)
             try:
                 out = format_func(format_, val)
                 if not isinstance(out, str):
-                    raise ValueError('Format function for value {} returned {} '
-                                     'instead of string type'
-                                     .format(val, type(val)))
+                    raise ValueError(
+                        f"Format function for value {val} returned {type(val)} "
+                        "instead of string type"
+                    )
             except Exception as err:
                 # For a masked element, the format function call likely failed
                 # to handle it.  Just return the string representation for now,
@@ -97,7 +101,7 @@ def get_auto_format_func(
                 if val is np.ma.masked:
                     return str(val)
 
-                raise ValueError(f'Format function for value {val} failed.') from err
+                raise ValueError(f"Format function for value {val} failed.") from err
             # If the user-supplied function handles formatting masked elements, use
             # it directly.  Otherwise, wrap it in a function that traps them.
             try:
@@ -117,15 +121,16 @@ def get_auto_format_func(
                     out = format_func(format_, val)
                     # Require that the format statement actually did something.
                     if out == format_:
-                        raise ValueError('the format passed in did nothing.')
+                        raise ValueError("the format passed in did nothing.")
                 except Exception:
                     continue
                 else:
                     break
             else:
                 # None of the possible string functions passed muster.
-                raise ValueError('unable to parse format string {} for its '
-                                 'column.'.format(format_))
+                raise ValueError(
+                    f"unable to parse format string {format_} for its column."
+                )
 
             # String-based format functions will fail on masked elements;
             # wrap them in a function that traps them.
@@ -143,6 +148,7 @@ def _get_pprint_include_names(table):
 
     These may be fnmatch unix-style globs.
     """
+
     def get_matches(name_globs, default):
         match_names = set()
         if name_globs:  # For None or () use the default
@@ -201,7 +207,7 @@ class TableFormatter:
             max_width = conf.max_width
 
         if max_lines is None or max_width is None:
-            lines, width = terminal_size()
+            width, lines = get_terminal_size()
 
         if max_lines is None:
             max_lines = lines
@@ -219,8 +225,17 @@ class TableFormatter:
 
         return max_lines, max_width
 
-    def _pformat_col(self, col, max_lines=None, show_name=True, show_unit=None,
-                     show_dtype=False, show_length=None, html=False, align=None):
+    def _pformat_col(
+        self,
+        col,
+        max_lines=None,
+        show_name=True,
+        show_unit=None,
+        show_dtype=False,
+        show_length=None,
+        html=False,
+        align=None,
+    ):
         """Return a list of formatted string representation of column values.
 
         Parameters
@@ -265,82 +280,92 @@ class TableFormatter:
             show_unit = col.info.unit is not None
 
         outs = {}  # Some values from _pformat_col_iter iterator that are needed here
-        col_strs_iter = self._pformat_col_iter(col, max_lines, show_name=show_name,
-                                               show_unit=show_unit,
-                                               show_dtype=show_dtype,
-                                               show_length=show_length,
-                                               outs=outs)
+        col_strs_iter = self._pformat_col_iter(
+            col,
+            max_lines,
+            show_name=show_name,
+            show_unit=show_unit,
+            show_dtype=show_dtype,
+            show_length=show_length,
+            outs=outs,
+        )
 
         # Replace tab and newline with text representations so they display nicely.
         # Newline in particular is a problem in a multicolumn table.
-        col_strs = [val.replace('\t', '\\t').replace('\n', '\\n') for val in col_strs_iter]
+        col_strs = [
+            val.replace("\t", "\\t").replace("\n", "\\n") for val in col_strs_iter
+        ]
         if len(col_strs) > 0:
             col_width = max(len(x) for x in col_strs)
 
         if html:
             from astropy.utils.xml.writer import xml_escape
-            n_header = outs['n_header']
+
+            n_header = outs["n_header"]
             for i, col_str in enumerate(col_strs):
                 # _pformat_col output has a header line '----' which is not needed here
                 if i == n_header - 1:
                     continue
-                td = 'th' if i < n_header else 'td'
-                val = f'<{td}>{xml_escape(col_str.strip())}</{td}>'
-                row = ('<tr>' + val + '</tr>')
+                td = "th" if i < n_header else "td"
+                val = f"<{td}>{xml_escape(col_str.strip())}</{td}>"
+                row = "<tr>" + val + "</tr>"
                 if i < n_header:
-                    row = ('<thead>' + row + '</thead>')
+                    row = "<thead>" + row + "</thead>"
                 col_strs[i] = row
 
             if n_header > 0:
                 # Get rid of '---' header line
                 col_strs.pop(n_header - 1)
-            col_strs.insert(0, '<table>')
-            col_strs.append('</table>')
+            col_strs.insert(0, "<table>")
+            col_strs.append("</table>")
 
         # Now bring all the column string values to the same fixed width
         else:
             col_width = max(len(x) for x in col_strs) if col_strs else 1
 
             # Center line header content and generate dashed headerline
-            for i in outs['i_centers']:
+            for i in outs["i_centers"]:
                 col_strs[i] = col_strs[i].center(col_width)
-            if outs['i_dashes'] is not None:
-                col_strs[outs['i_dashes']] = '-' * col_width
+            if outs["i_dashes"] is not None:
+                col_strs[outs["i_dashes"]] = "-" * col_width
 
             # Format columns according to alignment.  `align` arg has precedent, otherwise
             # use `col.format` if it starts as a legal alignment string.  If neither applies
             # then right justify.
-            re_fill_align = re.compile(r'(?P<fill>.?)(?P<align>[<^>=])')
+            re_fill_align = re.compile(r"(?P<fill>.?)(?P<align>[<^>=])")
             match = None
             if align:
                 # If there is an align specified then it must match
                 match = re_fill_align.match(align)
                 if not match:
-                    raise ValueError("column align must be one of '<', '^', '>', or '='")
+                    raise ValueError(
+                        "column align must be one of '<', '^', '>', or '='"
+                    )
             elif isinstance(col.info.format, str):
                 # col.info.format need not match, in which case rjust gets used
                 match = re_fill_align.match(col.info.format)
 
             if match:
-                fill_char = match.group('fill')
-                align_char = match.group('align')
-                if align_char == '=':
-                    if fill_char != '0':
+                fill_char = match.group("fill")
+                align_char = match.group("align")
+                if align_char == "=":
+                    if fill_char != "0":
                         raise ValueError("fill character must be '0' for '=' align")
-                    fill_char = ''  # str.zfill gets used which does not take fill char arg
+                    # str.zfill gets used which does not take fill char arg
+                    fill_char = ""
             else:
-                fill_char = ''
-                align_char = '>'
+                fill_char = ""
+                align_char = ">"
 
-            justify_methods = {'<': 'ljust', '^': 'center', '>': 'rjust', '=': 'zfill'}
+            justify_methods = {"<": "ljust", "^": "center", ">": "rjust", "=": "zfill"}
             justify_method = justify_methods[align_char]
             justify_args = (col_width, fill_char) if fill_char else (col_width,)
 
             for i, col_str in enumerate(col_strs):
                 col_strs[i] = getattr(col_str, justify_method)(*justify_args)
 
-        if outs['show_length']:
-            col_strs.append(f'Length = {len(col)} rows')
+        if outs["show_length"]:
+            col_strs.append(f"Length = {len(col)} rows")
 
         return col_strs, outs
 
@@ -354,12 +379,24 @@ class TableFormatter:
         if dtype is None or dtype.names is None:
             return name
 
-        structure = ', '.join([self._name_and_structure(name, dt, sep="")
-                               for name, (dt, _) in dtype.fields.items()])
+        structure = ", ".join(
+            [
+                self._name_and_structure(name, dt, sep="")
+                for name, (dt, _) in dtype.fields.items()
+            ]
+        )
         return f"{name}{sep}[{structure}]"
 
-    def _pformat_col_iter(self, col, max_lines, show_name, show_unit, outs,
-                          show_dtype=False, show_length=None):
+    def _pformat_col_iter(
+        self,
+        col,
+        max_lines,
+        show_name,
+        show_unit,
+        outs,
+        show_dtype=False,
+        show_length=None,
+    ):
         """Iterator which yields formatted string representation of column values.
 
         Parameters
@@ -387,8 +424,8 @@ class TableFormatter:
             if the column is not shown completely.
         """
         max_lines, _ = self._get_pprint_size(max_lines, -1)
-        dtype = getattr(col, 'dtype', None)
-        multidims = getattr(col, 'shape', [0])[1:]
+        dtype = getattr(col, "dtype", None)
+        multidims = getattr(col, "shape", [0])[1:]
         if multidims:
             multidim0 = tuple(0 for n in multidims)
             multidim1 = tuple(n - 1 for n in multidims)
@@ -407,19 +444,22 @@ class TableFormatter:
         if show_unit:
             i_centers.append(n_header)
             n_header += 1
-            yield str(col.info.unit or '')
+            yield str(col.info.unit or "")
         if show_dtype:
             i_centers.append(n_header)
             n_header += 1
             if dtype is not None:
-                col_dtype = dtype_info_name((dtype, multidims))
+                # For zero-length strings, np.dtype((dtype, ())) does not work;
+                # see https://github.com/numpy/numpy/issues/27301
+                # As a work-around, just omit the shape if there is none.
+                col_dtype = dtype_info_name((dtype, multidims) if multidims else dtype)
             else:
-                col_dtype = col.__class__.__qualname__ or 'object'
+                col_dtype = col.__class__.__qualname__ or "object"
             yield col_dtype
         if show_unit or show_name or show_dtype:
             i_dashes = n_header
             n_header += 1
-            yield '---'
+            yield "---"
 
         max_lines -= n_header
         n_print2 = max_lines // 2
@@ -453,10 +493,11 @@ class TableFormatter:
         # - get_auto_format_func() returns a wrapped version of auto_format_func
         #    with the column id and possible_string_format_functions as
         #    enclosed variables.
-        col_format = col.info.format or getattr(col.info, 'default_format',
-                                                None)
-        pssf = (getattr(col.info, 'possible_string_format_functions', None)
-                or _possible_string_format_functions)
+        col_format = col.info.format or getattr(col.info, "default_format", None)
+        pssf = (
+            getattr(col.info, "possible_string_format_functions", None)
+            or _possible_string_format_functions
+        )
         auto_format_func = get_auto_format_func(col, pssf)
         format_func = col.info._format_funcs.get(col_format, auto_format_func)
 
@@ -465,8 +506,9 @@ class TableFormatter:
                 show_length = True
             i0 = n_print2 - (1 if show_length else 0)
             i1 = n_rows - n_print2 - max_lines % 2
-            indices = np.concatenate([np.arange(0, i0 + 1),
-                                      np.arange(i1 + 1, len(col))])
+            indices = np.concatenate(
+                [np.arange(0, i0 + 1), np.arange(i1 + 1, len(col))]
+            )
         else:
             i0 = -1
             indices = np.arange(len(col))
@@ -484,31 +526,41 @@ class TableFormatter:
                 else:
                     left = format_func(col_format, col[(idx,) + multidim0])
                     right = format_func(col_format, col[(idx,) + multidim1])
-                    return f'{left} .. {right}'
+                    return f"{left} .. {right}"
             else:
                 return format_func(col_format, col[idx])
 
         # Add formatted values if within bounds allowed by max_lines
         for idx in indices:
             if idx == i0:
-                yield '...'
+                yield "..."
             else:
                 try:
                     yield format_col_str(idx)
                 except ValueError:
                     raise ValueError(
-                        'Unable to parse format string "{}" for entry "{}" '
-                        'in column "{}"'.format(col_format, col[idx],
-                                                col.info.name))
+                        f'Unable to parse format string "{col_format}" for '
+                        f'entry "{col[idx]}" in column "{col.info.name}"'
+                    )
 
-        outs['show_length'] = show_length
-        outs['n_header'] = n_header
-        outs['i_centers'] = i_centers
-        outs['i_dashes'] = i_dashes
+        outs["show_length"] = show_length
+        outs["n_header"] = n_header
+        outs["i_centers"] = i_centers
+        outs["i_dashes"] = i_dashes
 
-    def _pformat_table(self, table, max_lines=None, max_width=None,
-                       show_name=True, show_unit=None, show_dtype=False,
-                       html=False, tableid=None, tableclass=None, align=None):
+    def _pformat_table(
+        self,
+        table,
+        max_lines=None,
+        max_width=None,
+        show_name=True,
+        show_unit=None,
+        show_dtype=False,
+        html=False,
+        tableid=None,
+        tableclass=None,
+        align=None,
+    ):
         """Return a list of lines for the formatted string representation of
         the table.
 
@@ -573,12 +625,14 @@ class TableFormatter:
 
         elif isinstance(align, (list, tuple)):
             if len(align) != n_cols:
-                raise ValueError('got {} alignment values instead of '
-                                 'the number of columns ({})'
-                                 .format(len(align), n_cols))
+                raise ValueError(
+                    f"got {len(align)} alignment values instead of "
+                    f"the number of columns ({n_cols})"
+                )
         else:
-            raise TypeError('align keyword must be str or list or tuple (got {})'
-                            .format(type(align)))
+            raise TypeError(
+                f"align keyword must be str or list or tuple (got {type(align)})"
+            )
 
         # Process column visibility from table pprint_include_names and
         # pprint_exclude_names attributes and get the set of columns to show.
@@ -590,25 +644,30 @@ class TableFormatter:
             if col.info.name not in pprint_include_names:
                 continue
 
-            lines, outs = self._pformat_col(col, max_lines, show_name=show_name,
-                                            show_unit=show_unit, show_dtype=show_dtype,
-                                            align=align_)
-            if outs['show_length']:
+            lines, outs = self._pformat_col(
+                col,
+                max_lines,
+                show_name=show_name,
+                show_unit=show_unit,
+                show_dtype=show_dtype,
+                align=align_,
+            )
+            if outs["show_length"]:
                 lines = lines[:-1]
             cols.append(lines)
 
         if not cols:
-            return ['<No columns>'], {'show_length': False}
+            return ["<No columns>"], {"show_length": False}
 
         # Use the values for the last column since they are all the same
-        n_header = outs['n_header']
+        n_header = outs["n_header"]
 
         n_rows = len(cols[0])
 
         def outwidth(cols):
             return sum(len(c[0]) for c in cols) + len(cols) - 1
 
-        dots_col = ['...'] * n_rows
+        dots_col = ["..."] * n_rows
         middle = len(cols) // 2
         while outwidth(cols) > max_width:
             if len(cols) == 1:
@@ -628,11 +687,11 @@ class TableFormatter:
             from astropy.utils.xml.writer import xml_escape
 
             if tableid is None:
-                tableid = f'table{id(table)}'
+                tableid = f"table{id(table)}"
 
             if tableclass is not None:
                 if isinstance(tableclass, list):
-                    tableclass = ' '.join(tableclass)
+                    tableclass = " ".join(tableclass)
                 rows.append(f'<table id="{tableid}" class="{tableclass}">')
             else:
                 rows.append(f'<table id="{tableid}">')
@@ -641,23 +700,29 @@ class TableFormatter:
                 # _pformat_col output has a header line '----' which is not needed here
                 if i == n_header - 1:
                     continue
-                td = 'th' if i < n_header else 'td'
-                vals = (f'<{td}>{xml_escape(col[i].strip())}</{td}>'
-                        for col in cols)
-                row = ('<tr>' + ''.join(vals) + '</tr>')
+                td = "th" if i < n_header else "td"
+                vals = (f"<{td}>{xml_escape(col[i].strip())}</{td}>" for col in cols)
+                row = "<tr>" + "".join(vals) + "</tr>"
                 if i < n_header:
-                    row = ('<thead>' + row + '</thead>')
+                    row = "<thead>" + row + "</thead>"
                 rows.append(row)
-            rows.append('</table>')
+            rows.append("</table>")
         else:
             for i in range(n_rows):
-                row = ' '.join(col[i] for col in cols)
+                row = " ".join(col[i] for col in cols)
                 rows.append(row)
 
         return rows, outs
 
-    def _more_tabcol(self, tabcol, max_lines=None, max_width=None,
-                     show_name=True, show_unit=None, show_dtype=False):
+    def _more_tabcol(
+        self,
+        tabcol,
+        max_lines=None,
+        max_width=None,
+        show_name=True,
+        show_unit=None,
+        show_dtype=False,
+    ):
         """Interactive "more" of a table or column.
 
         Parameters
@@ -679,7 +744,7 @@ class TableFormatter:
         show_dtype : bool
             Include a header row for column dtypes. Default is False.
         """
-        allowed_keys = 'f br<>qhpn'
+        allowed_keys = "f br<>qhpn"
 
         # Count the header lines
         n_header = 0
@@ -693,10 +758,14 @@ class TableFormatter:
             n_header += 1
 
         # Set up kwargs for pformat call.  Only Table gets max_width.
-        kwargs = dict(max_lines=-1, show_name=show_name, show_unit=show_unit,
-                      show_dtype=show_dtype)
-        if hasattr(tabcol, 'columns'):  # tabcol is a table
-            kwargs['max_width'] = max_width
+        kwargs = dict(
+            max_lines=-1,
+            show_name=show_name,
+            show_unit=show_unit,
+            show_dtype=show_dtype,
+        )
+        if hasattr(tabcol, "columns"):  # tabcol is a table
+            kwargs["max_width"] = max_width
 
         # If max_lines is None (=> query screen size) then increase by 2.
         # This is because get_pprint_size leaves 6 extra lines so that in
@@ -715,49 +784,54 @@ class TableFormatter:
             i1 = i0 + delta_lines  # Last table/col row to show
             if showlines:  # Don't always show the table (e.g. after help)
                 try:
-                    os.system('cls' if os.name == 'nt' else 'clear')
+                    os.system("cls" if os.name == "nt" else "clear")
                 except Exception:
                     pass  # No worries if clear screen call fails
                 lines = tabcol[i0:i1].pformat(**kwargs)
-                colors = ('red' if i < n_header else 'default'
-                          for i in range(len(lines)))
+                colors = (
+                    "red" if i < n_header else "default" for i in range(len(lines))
+                )
                 for color, line in zip(colors, lines):
                     color_print(line, color)
             showlines = True
             print()
-            print("-- f, <space>, b, r, p, n, <, >, q h (help) --", end=' ')
+            print("-- f, <space>, b, r, p, n, <, >, q h (help) --", end=" ")
             # Get a valid key
             while True:
                 try:
                     key = inkey().lower()
                 except Exception:
                     print("\n")
-                    log.error('Console does not support getting a character'
-                              ' as required by more().  Use pprint() instead.')
+                    log.error(
+                        "Console does not support getting a character"
+                        " as required by more().  Use pprint() instead."
+                    )
                     return
                 if key in allowed_keys:
                     break
             print(key)
 
-            if key.lower() == 'q':
+            if key.lower() == "q":
                 break
-            elif key == ' ' or key == 'f':
+
+            if key == " " or key == "f":
                 i0 += delta_lines
-            elif key == 'b':
+            elif key == "b":
                 i0 = i0 - delta_lines
-            elif key == 'r':
+            elif key == "r":
                 pass
-            elif key == '<':
+            elif key == "<":
                 i0 = 0
-            elif key == '>':
+            elif key == ">":
                 i0 = len(tabcol)
-            elif key == 'p':
+            elif key == "p":
                 i0 -= 1
-            elif key == 'n':
+            elif key == "n":
                 i0 += 1
-            elif key == 'h':
+            elif key == "h":
                 showlines = False
-                print("""
+                print(
+                    """
     Browsing keys:
        f, <space> : forward one page
        b : back one page
@@ -767,7 +841,9 @@ class TableFormatter:
        < : go to beginning
        > : go to end
        q : quit browsing
-       h : print this help""", end=' ')
+       h : print this help""",
+                    end=" ",
+                )
             if i0 < 0:
                 i0 = 0
             if i0 >= len(tabcol) - delta_lines:
