@@ -27,6 +27,7 @@ from astropy.table import (
 from astropy.table.column import BaseColumn
 from astropy.table.serialize import represent_mixins_as_columns
 from astropy.table.table_helpers import ArrayWrapper
+from astropy.utils.compat import NUMPY_LT_2_0
 from astropy.utils.data_info import ParentDtypeInfo
 from astropy.utils.exceptions import AstropyUserWarning
 from astropy.utils.metadata import MergeConflictWarning
@@ -54,6 +55,7 @@ def test_attributes(mixin_cols):
             time.Time,
             time.TimeDelta,
             coordinates.BaseRepresentationOrDifferential,
+            coordinates.StokesCoord,
         ),
     ):
         with pytest.raises(AttributeError):
@@ -166,14 +168,59 @@ def test_io_time_write_fits_standard(tmp_path, table_types):
     filename = tmp_path / "table-tmp"
 
     # Show that FITS format succeeds
-    with pytest.warns(
-        AstropyUserWarning,
-        match=(
-            'Time Column "btai" has no specified location, '
-            "but global Time Position is present"
-        ),
-    ):
+    with pytest.warns(AstropyUserWarning) as record:
         t.write(filename, format="fits", overwrite=True)
+
+    # The exact sequence probably
+    # does not matter too much, so we'll just try to match the *set* of warnings
+    warnings = {wm.message.args[0] for wm in record}
+
+    expected = {
+        (
+            'Time Column "btai" has no specified location, '
+            "but global Time Position is present, "
+            "which will be the default for this column in FITS specification."
+        ),
+        (
+            'Time Column "btdb" has no specified location, '
+            "but global Time Position is present, "
+            "which will be the default for this column in FITS specification."
+        ),
+        (
+            'Time Column "btcg" has no specified location, '
+            "but global Time Position is present, "
+            "which will be the default for this column in FITS specification."
+        ),
+        (
+            'Time Column "btt" has no specified location, '
+            "but global Time Position is present, which will be the default "
+            "for this column in FITS specification."
+        ),
+        (
+            'Time Column "butc" has no specified location, '
+            "but global Time Position is present, "
+            "which will be the default for this column in FITS specification."
+        ),
+        (
+            'Earth Location "TOPOCENTER" for Time Column "atdb"'
+            ' is incompatible with scale "TDB".'
+        ),
+        (
+            'Earth Location "TOPOCENTER" for Time Column "atcb"'
+            ' is incompatible with scale "TCB".'
+        ),
+        (
+            'Time Column "but1" has no specified location, '
+            "but global Time Position is present, "
+            "which will be the default for this column in FITS specification."
+        ),
+        (
+            'Time Column "btcb" has no specified location, '
+            "but global Time Position is present, "
+            "which will be the default for this column in FITS specification."
+        ),
+    }
+    assert warnings == expected, f"Got some unexpected warnings\n{warnings - expected}"
     with pytest.warns(
         AstropyUserWarning,
         match='Time column reference position "TRPOSn" is not specified',
@@ -710,56 +757,104 @@ def test_quantity_representation():
     ]
 
 
-def test_representation_representation():
+@pytest.mark.parametrize(
+    "c, expected_pformat",
+    [
+        pytest.param(
+            coordinates.CartesianRepresentation([0], [1], [0], unit=u.one),
+            # With no unit we get "None" in the unit row
+            [
+                "    col0    ",
+                "------------",
+                "(0., 1., 0.)",
+            ]
+            if NUMPY_LT_2_0
+            else [
+                "      col0     ",
+                "---------------",
+                "(0.0, 1.0, 0.0)",
+            ],
+            id="cartesian_wo_unit",
+        ),
+        pytest.param(
+            coordinates.CartesianRepresentation([0], [1], [0], unit="m"),
+            [
+                "    col0    ",
+                "     m      ",
+                "------------",
+                "(0., 1., 0.)",
+            ]
+            if NUMPY_LT_2_0
+            else [
+                "      col0     ",
+                "       m       ",
+                "---------------",
+                "(0.0, 1.0, 0.0)",
+            ],
+            id="cartesian_w_unit",
+        ),
+        pytest.param(
+            coordinates.SphericalRepresentation([10] * u.deg, [20] * u.deg, [1] * u.pc),
+            [
+                "     col0     ",
+                " deg, deg, pc ",
+                "--------------",
+                "(10., 20., 1.)",
+            ]
+            if NUMPY_LT_2_0
+            else [
+                "       col0      ",
+                "   deg, deg, pc  ",
+                "-----------------",
+                "(10.0, 20.0, 1.0)",
+            ],
+            id="spherical",
+        ),
+        pytest.param(
+            coordinates.UnitSphericalRepresentation([10] * u.deg, [20] * u.deg),
+            [
+                "   col0   ",
+                "   deg    ",
+                "----------",
+                "(10., 20.)",
+            ]
+            if NUMPY_LT_2_0
+            else [
+                "    col0    ",
+                "    deg     ",
+                "------------",
+                "(10.0, 20.0)",
+            ],
+            id="unitspherical",
+        ),
+        pytest.param(
+            coordinates.SphericalCosLatDifferential(
+                [10] * u.mas / u.yr, [2] * u.mas / u.yr, [10] * u.km / u.s
+            ),
+            [
+                "           col0           ",
+                "mas / yr, mas / yr, km / s",
+                "--------------------------",
+                "            (10., 2., 10.)",
+            ]
+            if NUMPY_LT_2_0
+            else [
+                "           col0           ",
+                "mas / yr, mas / yr, km / s",
+                "--------------------------",
+                "         (10.0, 2.0, 10.0)",
+            ],
+            id="sphericalcoslatdifferential",
+        ),
+    ],
+)
+@pytest.mark.usefixtures("without_legacy_printoptions")
+def test_representation_representation(c, expected_pformat):
     """
     Test that Representations are represented correctly.
     """
-    # With no unit we get "None" in the unit row
-    c = coordinates.CartesianRepresentation([0], [1], [0], unit=u.one)
     t = Table([c])
-    assert t.pformat() == [
-        "    col0    ",
-        "------------",
-        "(0., 1., 0.)",
-    ]
-
-    c = coordinates.CartesianRepresentation([0], [1], [0], unit="m")
-    t = Table([c])
-    assert t.pformat() == [
-        "    col0    ",
-        "     m      ",
-        "------------",
-        "(0., 1., 0.)",
-    ]
-
-    c = coordinates.SphericalRepresentation([10] * u.deg, [20] * u.deg, [1] * u.pc)
-    t = Table([c])
-    assert t.pformat() == [
-        "     col0     ",
-        " deg, deg, pc ",
-        "--------------",
-        "(10., 20., 1.)",
-    ]
-
-    c = coordinates.UnitSphericalRepresentation([10] * u.deg, [20] * u.deg)
-    t = Table([c])
-    assert t.pformat() == [
-        "   col0   ",
-        "   deg    ",
-        "----------",
-        "(10., 20.)",
-    ]
-
-    c = coordinates.SphericalCosLatDifferential(
-        [10] * u.mas / u.yr, [2] * u.mas / u.yr, [10] * u.km / u.s
-    )
-    t = Table([c])
-    assert t.pformat() == [
-        "           col0           ",
-        "mas / yr, mas / yr, km / s",
-        "--------------------------",
-        "            (10., 2., 10.)",
-    ]
+    assert t.pformat() == expected_pformat
 
 
 def test_skycoord_representation():
@@ -805,6 +900,7 @@ def test_skycoord_representation():
 
 
 @pytest.mark.parametrize("as_ndarray_mixin", [True, False])
+@pytest.mark.usefixtures("without_legacy_printoptions")
 def test_ndarray_mixin(as_ndarray_mixin):
     """
     Test directly adding various forms of structured ndarray columns to a table.
@@ -812,7 +908,7 @@ def test_ndarray_mixin(as_ndarray_mixin):
     (which provides full support for structured array Column's). This test shows
     that the end behavior is the same in both cases.
     """
-    a = np.array([(1, "a"), (2, "b"), (3, "c"), (4, "d")], dtype="<i4," + "|U1")
+    a = np.array([(1, "a"), (2, "b"), (3, "c"), (4, "d")], dtype="<i4,|U1")
     b = np.array(
         [(10, "aa"), (20, "bb"), (30, "cc"), (40, "dd")],
         dtype=[("x", "i4"), ("y", "U2")],
@@ -870,15 +966,28 @@ def test_ndarray_mixin(as_ndarray_mixin):
     assert t[1]["d"][0] == d[1][0]
     assert t[1]["d"][1] == d[1][1]
 
-    assert t.pformat(show_dtype=True) == [
-        "  a [f0, f1]     b [x, y]      c [rx, ry]      d    ",
-        "(int32, str1) (int32, str2) (float64, str3) int64[2]",
-        "------------- ------------- --------------- --------",
-        "     (1, 'a')    (10, 'aa')   (100., 'raa')   0 .. 1",
-        "     (2, 'b')    (20, 'bb')   (200., 'rbb')   2 .. 3",
-        "     (3, 'c')    (30, 'cc')   (300., 'rcc')   4 .. 5",
-        "     (4, 'd')    (40, 'dd')   (400., 'rdd')   6 .. 7",
-    ]
+    assert (
+        t.pformat(show_dtype=True)
+        == [
+            "  a [f0, f1]     b [x, y]      c [rx, ry]      d    ",
+            "(int32, str1) (int32, str2) (float64, str3) int64[2]",
+            "------------- ------------- --------------- --------",
+            "     (1, 'a')    (10, 'aa')   (100., 'raa')   0 .. 1",
+            "     (2, 'b')    (20, 'bb')   (200., 'rbb')   2 .. 3",
+            "     (3, 'c')    (30, 'cc')   (300., 'rcc')   4 .. 5",
+            "     (4, 'd')    (40, 'dd')   (400., 'rdd')   6 .. 7",
+        ]
+        if NUMPY_LT_2_0
+        else [
+            "  a [f0, f1]     b [x, y]      c [rx, ry]      d    ",
+            "(int32, str1) (int32, str2) (float64, str3) int64[2]",
+            "------------- ------------- --------------- --------",
+            "     (1, 'a')    (10, 'aa')  (100.0, 'raa')   0 .. 1",
+            "     (2, 'b')    (20, 'bb')  (200.0, 'rbb')   2 .. 3",
+            "     (3, 'c')    (30, 'cc')  (300.0, 'rcc')   4 .. 5",
+            "     (4, 'd')    (40, 'dd')  (400.0, 'rdd')   6 .. 7",
+        ]
+    )
 
 
 def test_possible_string_format_functions():
